import struct, re, os, os.path
from world_util import *
	
def readItem(data, startIdx):
	#First read the name (NUL-terminated string)
	nameEnd = data.find('\0', startIdx)
	if nameEnd == -1: 
		return (None, -1)
	name = data[startIdx:nameEnd]
	
	#Then read the length of the data (4 byte little-endian int)
	itemLen = struct.unpack('I', data[nameEnd + 1:nameEnd + 5])[0]
	itemStart = nameEnd + 5
	
	#Read the data
	itemContent = data[itemStart:itemStart+itemLen]
	
	#Parse the name into coord & type in case we want to do any extra processing
	nameCoord, nameType = name.split(' ')
	
	return ((nameCoord, nameType, itemContent), itemStart+itemLen)

def readAllItems(data):
	items = []
	nextIdx = 0
	#Keep parsing until we run out of data
	while True:
		item, nextIdx = readItem(data, nextIdx)
		if item != None:
			items.append(item)
		else:
			break
	
	return items

def unpackWorldBin(filename, pathToExtractTo):
	"""Example Usage:
	    unpackWorldBin("c:/foo/World.bin", "c:/foo/extracted/")
		
	"""
	data = fileReadAllBytes(filename)
	items = readAllItems(data)
	
	for item in items:
		# Skip if file is empty. Note: This causes issues when rebuilding World.bin
		# if len(item[2].strip()) == 0: continue
		extn = item[1] == "Script" and "py" or "ini"
		filename = pathToExtractTo + "/{0} {1}.{2}".format(item[0], item[1], extn)
		fileWriteAllBytes(filename, item[2])
		
def writeItem(buf, item):
	# Write the name
	buf.extend("{0} {1}\0".format(item[0], item[1]))
	# Write the data length
	buf.extend(struct.pack("I", len(item[2])))
	# Write the data
	buf.extend(item[2])
	
def writeAllItems(items):
	buf = bytearray()
	for item in items:
		writeItem(buf, item)
	return buf
	
def packWorldBin(filename, pathToRead):
	files = os.listdir(pathToRead)
	#Match filenames against expected format
	fileMatchs = (re.match('(\d+,\d+) ([^.]*)\.(?:py|ini|txt)', f) 
				  for f in files)
	#Read matching files and unpack into (coord, type, data) structure
	items = ((m.group(1), 
			  m.group(2), 
			  fileReadAllBytes(os.path.join(pathToRead, m.group(0)))
			  ) for m in fileMatchs if m)
	#Write 
	data = writeAllItems(items)
	fileWriteAllBytes(filename, data)
		

if __name__ == '__main__':
	unpackWorldBin("World.bin", "extracted/")
	
	
	
	
	
	
	
	
	
	
	
	
	