import os.path, re

from collections import namedtuple
from ConfigParser import RawConfigParser
from PIL import Image
from cStringIO import StringIO

from world_util import *
from world_unpack import readAllItems

#Cell = namedtuple('Cell', ['x','y','data'])
LevelData = namedtuple('LevelData', ['x','y','tile_grid','tilesets'])
rez = 8 #scale of one tile in one level on rendered map

def loadLevel(coord, data):
	x,y = (int(n) for n in coord.split(","))
	text_data = RawConfigParser()
	text_data.readfp(StringIO(data))
	
	#Load the raw text of the rows
	text_rows = text_data.items("Tile Map")
	text_rows.sort(key=lambda kv: int(kv[0]))
	
	#Get dimensions of the grid
	maxX = max(len(data) // 3 for r, data in text_rows)
	maxY = max(int(r) + 1 for r, data in text_rows if r.isdigit())
	
	if maxX != 31 or maxY != 18:
		maxX, maxY = 31, 18
	
	#Load tiles into the grid
	grid = [[None for c in xrange(maxX)] for r in xrange(maxY)]
	for r, row_data in text_rows:
		if not r.isdigit(): continue
		for c in xrange(len(row_data) // 3):
			cell = row_data[c*3:c*3+3]
			if not re.match('[0-2][0-4][0-4]', cell): continue
			grid[int(r)][c] = cell
	
	#Get the tile sets
	tilesets = (
		text_data.get("General", "Tileset 0"),
		text_data.get("General", "Tileset 1"),
		text_data.get("General", "Tileset 2"),
	)
	
	#Get the background color
	#bg = text_data.getint("General", "Tileset Background")
	#background color is useless - it's always #808080 
			
	#rows = [[row[c*3:c*3+3] for c in xrange(len(row) // 3)] 
	#		for i, row in text_rows]
	
	return LevelData(x, y, grid, tilesets)

def loadLevels(filename="World.bin"):
	items = readAllItems(fileReadAllBytes("World.bin"))
	
	return [loadLevel(item[0], item[2]) for item in items if item[1] == "Tiles"]
	
def loadTilesets(dir="c:/program files (x86)/nifflas/knytt underground/world/masks/"):
	
	images = [(os.path.splitext(f)[0], Image.open(os.path.join(dir, f)))
			  for f in os.listdir(dir) if f.endswith(".png")]
	
	tilesets = dict()
	for name, img in images:
		img = img.convert("L")
		tiles = dict()
		for c in xrange(5):
			for r in xrange(5):
				mask = img.crop((c*44+6, r*44+6, c*44+38, r*44+38))
				tiles["{}{}".format(c,r)] = mask.resize((rez,rez), Image.ANTIALIAS)
		tilesets[name] = tiles
		
	return tilesets
	

	
def drawLevel(level, tilesets_lookup):
	#Generate a color based on the type of tile set used (later use music?)
	rgbahash = hash(level.tilesets[0])
	bgcolor = "rgb({},{},{})".format(
		64 + ((rgbahash & 0x00FF0000) >> 16) * 192 // 256,
		64 + ((rgbahash & 0x0000FF00) >> 8) * 192 // 256,
		64 + ((rgbahash & 0x000000FF) >> 0) * 192 // 256,
	)

	#Make an image
	grid = level.tile_grid
	grid_w = len(grid[0])
	grid_h = len(grid)
	img = Image.new("RGB", (grid_w * rez, grid_h * rez), bgcolor)
	
	#Look up tile sets
	tilesets = []
	for ts in level.tilesets:
		#if ts != "Null Tileset" and not tilesets_lookup.has_key(ts):
		#	print "Level {},{}, Tileset {} - Unrecognized Tileset".format(level.x, level.y, ts)
		
		tilesets.append(tilesets_lookup.get(ts))
	
	
	#Copy tiles to image
	for x in xrange(grid_w):
		for y in xrange(grid_h):
			cell = grid[y][x]
			if not cell: continue
			if not cell.isdigit(): continue
			tileset = tilesets[int(cell[0])]
			if not tileset: 
				#Tileset isn't loaded. Can't render tile. 
				#This usually happens when we use files from the Masks/ directory instead of Tilesets/
				continue
			
			tile = tileset.get(cell[1:3])
			if not tile:
				print "Level {},{}, Cell {},{}, Tile {} - Tile not found".format(level.x, level.y, x, y, cell)
				continue
	
			img.paste("rgb(0,0,0)", (x*rez, y*rez, (x+1)*rez, (y+1)*rez), tile)
	return img


def makemap():
	levels = loadLevels()
	tilesets = loadTilesets()

	maxX = max(level.x for level in levels)
	maxY = max(level.y for level in levels)

	map_img = Image.new("RGB", (maxX * 31 * rez, maxY * 18 * rez), "rgb({},{},{})".format(32,32,32))
	for level in levels:
		x = level.x * 31 * rez
		y = level.y * 18 * rez
		w = 31 * rez
		h = 18 * rez
		level_img = drawLevel(level, tilesets)
		map_img.paste(level_img, (x,y,x+w,y+h))
			
	map_img.save("map.png")
		
